import path from 'path'
import webpack from "webpack";

import FriendlyErrorsPlugin from "friendly-errors-webpack-plugin";
import OptimizeCssAssetsPlugin from "optimize-css-assets-webpack-plugin";
import MiniCssExtractPlugin from "mini-css-extract-plugin";
import { VueLoaderPlugin } from "vue-loader";
import TerserPlugin from "terser-webpack-plugin";

import options from "./config/options";

import { searchEntries } from "./utils";

const vue3Dir = path.resolve('./js')

const vue3_entry = searchEntries({
  fileName: "index",
  entryPath: vue3Dir,
  fileNamePrefix: `vue3/js/`,
  isFuzzy: true,
});

// 基础配置
const config = {
  watch: options.__DEV__,
  watchOptions: {
    ignored: /node_modules/,
  },
  mode: process.env.NODE_ENV,
  entry: vue3_entry,
  output: Object.assign(options.output, {
    filename: "[name].js",
  }),
  externals: options.externals,
  resolve: {
    alias: {
      common: path.resolve('../', 'app/Resources/static-src/common'),
      'vue$': 'vue/dist/vue.esm-bundler.js'
    },
    extensions: ["*", ".js", ".jsx", ".vue"],
  },
  module: {
    rules: [
      {
        test: /\.vue$/,
        loader: "vue-loader",
      },
      {
        test: /\.js[x]?$/,
        use: [
          {
            loader: "thread-loader",
            options: {
              workers: options.cpuNumber,
            },
          },
          {
            loader: "babel-loader",
          },
        ],
        exclude: [path.resolve('./', 'node_modules')],
      },
      {
        test: /\.css$/,
        use: [
          options.__DEV__ ? 'vue-style-loader' : MiniCssExtractPlugin.loader,
          { loader: 'css-loader', options: { esModule: false }},
          'postcss-loader'
        ]
      },
      {
        test: /\.less$/,
        use: [
          options.__DEV__ ? 'vue-style-loader' : MiniCssExtractPlugin.loader,
          'css-loader',
          'postcss-loader',
          'less-loader'
        ]
      }
    ],
  },
  optimization: {
    minimizer: [new TerserPlugin()],
    splitChunks: {
      cacheGroups: {
        common: {
          name: `vue3/js/${options.commonsChunkFileName}`,
          chunks: "initial", //入口处开始提取代码
          minSize: 300000, //代码最小多大，进行抽离
          minChunks: 6,
        },
      },
    },
  },
  plugins: [
    new MiniCssExtractPlugin({
      filename: "[name].css",
      chunkFilename: "[id].css",
    }),
    new webpack.DefinePlugin({
      // __webpack_public_path__: `window.__publicPath`,
      "process.env": {
        NODE_ENV: JSON.stringify(process.env.NODE_ENV || "development"),
      },
    }),
    // new webpack.ProvidePlugin(options.global),
    new webpack.ContextReplacementPlugin(
      /moment[\\\/]locale$/,
      /^\.\/(zh-cn|en-gb)+\.js$/
    ),
    new VueLoaderPlugin(),
  ],
};


config.plugins = config.plugins.concat(new FriendlyErrorsPlugin());

if (process.env.NODE_ENV === 'production') {
  config.plugins = config.plugins.concat(new OptimizeCssAssetsPlugin());
} else {
  config.devtool = 'source-map';
}

export default config;
